unit MButton;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  ExtCtrls, Forms, Dialogs;

type
  TButtonLayout = (blGradientCaption,blCaptionCenter, blCaptionLeft, blCaptionRight, blCaptionTop, blCaptionBottom);
  TBorderWidth = 1..10;  
  TMargin = 3..25;
  TNumGlyphs = 1..3;
  TStyle = (stIE, stGradient, st98, stTripple);

  TMButton = class(TGraphicControl)
  private
    { Private declarations }
    Clicked:Boolean;
    fsb:boolean;
    F3d:Boolean;
    FFsc:TColor;
    FStyle:TStyle;
    FBeginClr : TColor;
    FEndClr : TColor;
    FStretch:Boolean;
    FBorderWidth: TBorderWidth;
    FButtonLayout: TButtonLayout;
    FCaption: TCaption;
    FClick,
    FEnter,
    FLeave,
    FGlyph: TBitmap;
    FMargin: TMargin;
    FNumGlyphs: TNumGlyphs;
    FTransparentColor: TColor;

    BtnRect, BtnRect1: TRect;
    HasMouse: boolean;

    procedure setfsb(c:boolean);
    procedure setfsc(c:TColor);
    procedure SetBeginClr(Value : TColor);
    procedure SetEndClr(Value : TColor);
    procedure SetStyle(c:TStyle);
    procedure setStretch(c:boolean);
    procedure setf3d(c:boolean);
    procedure SetBorderWidth(Value: TBorderWidth);
    procedure SetButtonLayOut(Value: TButtonLayout);
    procedure SetGlyph(Value: TBitmap);
    procedure SetEnter(Value: TBitmap);
    procedure SetClick(Value: TBitmap);
    procedure SetLeave(Value: TBitmap);
    procedure SetMargin(Value: TMargin);
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure SetText(Value: TCaption);
    procedure SetTransparentColor(Value: TColor);
    procedure InitBtnRect;
    procedure DrawCaption;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure FillHorizontal(FBeginClr,FEndClr:integer);
  published
    { Published declarations }
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 1;
    property ButtonLayout: TButtonLayout read FButtonLayout write SetButtonLayout default blCaptionCenter;
    property Caption: TCaption read FCaption write SetText;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property BitMaPEnter: TBitmap read FEnter write SetEnter;
    property BitMaPLeave: TBitmap read FLeave write SetLeave;
    property BitMaPClick: TBitmap read FClick write SetClick;
    property Margin: TMargin read FMargin write SetMargin default 5;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs default 2;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    property Style:TStyle read FStyle write SetStyle default st98;
    property Stretch:boolean read FStretch write SetStretch default false;
    property Font3d:boolean read F3d write SetF3D default false;
    property SideBlock:boolean read fsb write SetFsb default true;
    property Color;
    property Enabled;
    property Font;
    property Name;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property BeginColor : TColor read FBeginClr write SetBeginClr default clYellow;
    property EndColor : TColor read FEndClr write SetEndClr default clBtnFace;
    property FontShadowColor : TColor read FFSC write SetFSC default clWhite;
  end;

procedure Register;

implementation

procedure TMButton.InitBtnRect;
begin
  with self do begin
    BtnRect := rect(0,0,width,height);
    BtnRect1 := rect(1,1,width-1,height-1);
  end;
end;

procedure TMButton.DrawCaption;
var
  TopLeft: TPoint;
  fc:TColor;

begin

  with Canvas do
  begin
    Font.Color := Self.Font.Color;
    Font.Name := Self.Font.Name;
    Font.Size := Self.Font.Size;
    if not hasmouse
      then Font.Style := Self.Font.Style
      else font.style:=[fsBold];
    Brush.Style := bsClear;
  end;

  case ButtonLayout of
    blCaptionCenter: begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
    blCaptionTop:    begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := FMargin;
                     end;
    blCaptionBottom: begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) div 2;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) - FMargin;
                     end;
    blCaptionLeft:   begin
                       TopLeft.X := FMargin;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
    blGradientCaption:begin
                       TopLeft.X := FMargin+15;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;
    blCaptionRight:  begin
                       TopLeft.X := (Width - Canvas.TextWidth(Caption)) - FMargin;
                       TopLeft.Y := (Height - Canvas.TextHeight(Caption)) div 2;
                     end;

  end;

  if Caption <> '' then
  with Canvas do
  begin
     if Enabled then begin
        fc:=Font.Color;
        if Font3d then begin
          Font.Color:=ffsc;
          TextOut(TopLeft.X+1, TopLeft.Y+1, Caption);
        end;
        Font.Color:=fc;
        TextOut(TopLeft.X, TopLeft.Y, Caption);
     end
     else
     begin
       Font.Color := clWhite;
       TextOut(TopLeft.X + 1, TopLeft.Y + 1, Caption);
       Font.Color := clGrayText;
       TextOut(TopLeft.X, TopLeft.Y, Caption);
     end;

  end;
end;

procedure TMButton.SetBorderWidth(Value: TBorderWidth);
begin
  if (Value <> FBorderWidth) and (Value >= 1) and (Value < 10) then
  begin
    FBorderWidth := Value;
    SetMargin(Value + 2);
    invalidate;
  end;
end;

procedure TMButton.SetButtonLayout(Value: TButtonLayout);
begin
  if Value <> FButtonLayout then
  begin
     FButtonLayout := Value;
     invalidate;
  end;
end;

procedure TMButton.SetGlyph(Value: TBitmap);
begin
  FGlyph.Assign(Value);
end;
procedure TMButton.SetEnter(Value: TBitmap);
begin
  FEnter.Assign(Value);
end;

procedure TMButton.SetBeginClr(Value : TColor);
begin
  if FBeginClr <> Value then begin
    FBeginClr := Value;
    Invalidate;
  end;
end;

procedure TMButton.SetEndClr(Value : TColor);
begin
  if FEndClr <> Value then begin
    FEndClr := Value;
    Invalidate;
  end;
end;

procedure TMButton.SetClick(Value: TBitmap);
begin
  FClick.Assign(Value);
end;

procedure TMButton.SetLeave(Value: TBitmap);
begin
  FLeave.Assign(Value);
end;


procedure TMButton.SetMargin(Value: TMargin);
begin
  if (Value <> FMargin) and (Value > FBorderWidth + 1) then
  begin
    FMargin := Value;
    invalidate
  end;
end;

procedure TMButton.SetNumGlyphs(Value: TNumGlyphs);
begin
  if (Value <> FNumGlyphs) and (Value <= 3) and (Value > 0) then
  begin
    FNumGlyphs := Value;
    invalidate;
  end;
end;

procedure TMButton.SetText(Value: TCaption);
begin
  if Value <> FCaption then FCaption := Value;
  invalidate;
end;

procedure TMButton.SetTransparentColor(Value: TColor);
begin
  if Value <> FTransparentColor then
  begin
    FTransparentColor := Value;
    invalidate;
  end;
end;

constructor TMButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fsb:=true;
  F3d:=false;
  FBeginClr:=ClYellow;
  FEndClr:=ClBtnFace;
  FFsc:=ClWhite;
  clicked:=False;
  style:=st98;
  BorderWidth := 1;
  ButtonLayout := blCaptionCenter;
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clBtnFace;
  canvas.font.name:='Arial';
  FGlyph := TBitmap.Create;
  FEnter := TBitmap.create;
  FLeave := TBitMap.Create;
  FClick := TbitMap.Create;
  Font := Canvas.Font;
  HasMouse := false;
  Height := 25;
  Margin := 5;
  FNumGlyphs := 2;
  Width := 25;
end;

destructor TMButton.Destroy;
begin
  FGlyph.Free;
  FEnter.Free;
  FLeave.Free;
  FClick.Free;
  inherited Destroy;
end;

procedure TMButton.CMMouseEnter(var Message: TMessage);
var ERect:TRect;
begin
  HasMouse := true;
  if Enabled then
  begin
    InitBtnRect;
    invalidate;
    Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth);
  end;
end;

procedure TMButton.CMMouseLeave(var Message: TMessage);
begin
  HasMouse := false;
  if Enabled then
  begin
    InitBtnRect;
    invalidate;
    Frame3D(Canvas, BtnRect1, color,color,FBorderWidth);
  end;
end;

procedure TMButton.Paint;
var
  R,GlyphRect, BitmapRect: TRect;
  TopLeft: TPoint;
  PartGlyphWidth: integer;
begin
  InitBtnRect;

  with Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := Color;
    Pen.Color := clBtnFace;
  end;

  {
  Canvas.Rectangle(0, 0, Width, Height);
  }

  if (csDesigning in ComponentState) then
  begin
    InitBtnRect;
    Frame3D(Canvas, BtnRect, clBtnHighLight, clBtnShadow, FBorderWidth);
  end;

    case ButtonLayout of
      blCaptionCenter: begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
      blCaptionTop:    begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := (Self.Height - FGlyph.Height) - Margin;
                       end;
      blCaptionBottom: begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) div 2;
                         TopLeft.Y := Margin;
                       end;
      blCaptionLeft:   begin
                         TopLeft.X := (Self.Width - (FGlyph.Width div NumGlyphs)) - Margin;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
      blCaptionRight:  begin
                         TopLeft.X := Margin;
                         TopLeft.Y := (Self.Height - FGlyph.Height) div 2;
                       end;
    end;

  GlyphRect := Rect(TopLeft.X, TopLeft.Y, TopLeft.X + (FGlyph.Width div NumGlyphs), TopLeft.Y + FGlyph.Height);

  PartGlyphWidth := FGlyph.Width div NumGlyphs;

  if Enabled and HasMouse then
  case style of
    stgradient:begin
      FillHorizontal(FBeginClr,FEndClr);
      with canvas do if sideblock then begin
        pen.width:=0;
        brush.Style:=bsSolid;
        pen.mode:=pmCopy;
        pen.color:=FFSC;
        brush.color:=FFsc;
        rectangle(0,0,10,height);
        MoveTo(0,height);
        LineTo(width,height);
      end;
    end;
    sttripple:begin
      if Clicked then
      begin
        if Stretch then begin
          R:=Rect(0,0,Width,Height);
          Canvas.Stretchdraw(R,FClick);
        end else Canvas.draw(0,0,FClick);
      end else if Stretch then
      begin
        R:=Rect(0,0,Width,Height);
        Canvas.Stretchdraw(R,FEnter);
      end else Canvas.draw(0,0,FEnter);
    end;
    st98:begin
       BitmapRect := Rect(0, 0, PartGlyphWidth, FGlyph.Height);
       Frame3D(Canvas, BtnRect, clWhite, clBlack, FBorderWidth);
       Frame3D(Canvas, BtnRect1, clWhite, clBlack, FBorderWidth);
    end;
    stIE:begin
       BitmapRect := Rect(0, 0, PartGlyphWidth, FGlyph.Height);
       Frame3D(Canvas, BtnRect, clwhite, clGray, FBorderWidth);
    end;
  end;

  if Enabled and not HasMouse then
  case style of
    stgradient:with canvas do if sideblock then begin
      Brush.color:=FBeginClr;
      pen.color:=FbeginClr;
      pen.width:=2;
      rectangle(0,0,10,Height);
      MoveTo(0,height);
      lineto(width,height);
    end;
    sttripple:begin
      if stretch then begin
        R:=Rect(0,0,Width,Height);
        Canvas.Stretchdraw(R,FLeave);
      end else Canvas.draw(0,0,FLeave);
    end;
    st98:begin
      BitmapRect := Rect(FGlyph.Width - PartGlyphWidth, 0, PartGlyphWidth * NumGlyphs, FGlyph.Height);
      Frame3D(Canvas, BtnRect1, Color, Color, FBorderWidth);
      Frame3D(Canvas, BtnRect, clWhite, clBlack, FBorderWidth);
    end;
    stIE:begin
      BitmapRect := Rect(FGlyph.Width - PartGlyphWidth, 0, PartGlyphWidth * NumGlyphs, FGlyph.Height);
      Frame3D(Canvas, BtnRect, Color, Color, FBorderWidth);
    end;
  end;

  if not Enabled and (FNumGlyphs > 1) then
     BitmapRect := Rect(PartGlyphWidth, 0, PartGlyphWidth * 2, FGlyph.Height);
  if not Enabled and (FNumGlyphs = 1) then
     BitmapRect := Rect(0, 0, PartGlyphWidth, FGlyph.Height);

  Canvas.BrushCopy(GlyphRect, FGlyph, BitmapRect, TransparentColor);

  DrawCaption;
end;

procedure TMButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var r:TRect;
begin
  if Enabled then begin
    InitBtnRect;
    case style of
      st98:begin
        Frame3D(Canvas, BtnRect1, clBlack, clWhite, FBorderWidth);
        Frame3D(Canvas, BtnRect, clGray, clBlack, FBorderWidth);
      end;
      stIE:begin
        Frame3D(Canvas, BtnRect, clBlack, clWhite, FBorderWidth);
        Frame3D(Canvas, BtnRect1, clGray, clBlack, FBorderWidth);
      end;
      stTripple:begin
        Clicked:=True;
        invalidate;
      end;
      stgradient:begin
        FillHorizontal(FEndClr,FBeginClr);
        DrawCaption;
      end;
    end;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TMButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled then
  begin
    InitBtnRect;
    if HasMouse then
    case style of
      stGradient:begin
       invalidate;
      end;
      stTripple:begin
         Clicked:=False;
         invalidate;
      end;
      st98:begin
         Frame3D(Canvas, BtnRect, clwhite, clGray, FBorderWidth);
         Frame3D(Canvas, BtnRect1, clWhite, clBlack, FBorderWidth);
      end;
      stIE:begin
        Frame3D(Canvas, BtnRect1, Color, Color, FBorderWidth);
        Frame3D(Canvas, BtnRect, clwhite, clGray, FBorderWidth);
      end;
    end else case style of
      stTripple:;
      else Frame3D(Canvas, BtnRect, Color, Color, FBorderWidth);
    end;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure tmbutton.setStretch(c:boolean);
begin
  fStretch:=c;
  invalidate;
end;

procedure tmbutton.setstyle(c:tstyle);
begin
  fstyle:=c;
  case c of
    stGradient:ButtonLayOut:=blGradientCaption;
  end;
  invalidate;
end;

procedure tmbutton.setfsc(c:TColor);
begin
  ffsc:=c;
  invalidate;
end;

procedure tmbutton.setf3d(c:boolean);
begin
  f3d:=c;
  invalidate
end;

procedure tmbutton.setfsb(c:boolean);
begin
  fsb:=c;
  invalidate
end;


procedure Register;
begin
  RegisterComponents('miks', [TMButton]);
end;

procedure TMButton.FillHorizontal(FBeginClr,FEndClr:integer);
var
  fr,fg,fb,dr,dg,db:integer;
  ColorRect: TRect;
  I: Integer;
  d, R, G, B : Byte;
begin
  FR := FBeginClr and $000000ff;
  FG := (FBeginClr shr 8) and $000000ff;
  FB := (FBeginClr shr 16) and $000000ff;

  DR := (FEndClr and $000000ff) - FR;
  DG := ((FEndClr shr 8) and $000000ff) - FG;
  DB := ((FEndClr shr 16) and $000000ff) - FB;

  ColorRect.Top:= 0;
  ColorRect.Bottom := self.Height;
  if SideBlock then d:=15 else d:=0;
  for I := 0 to 255 do begin
    ColorRect.Left:= MulDiv (I, width, 256);
    ColorRect.Right:= MulDiv (I + 1, width, 256);
    R := fr + MulDiv(I, dr, 255);
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    with Canvas do begin
      Brush.Color := RGB(R, G, B);
      if colorrect.left>d then Canvas.FillRect(ColorRect);
    end;
  end;
end;


end.